@@ -12,7 +12,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <time.h>
 #include <unistd.h>
 #include <assert.h>
 #include <wchar.h>
@@ -61,24 +60,20 @@ int asprintf(char **string_ptr, const char *format, ...)
 
     va_start(arg, format);
     size = vsnprintf(NULL, 0, format, arg);
-    size++;
     va_end(arg);
-    va_start(arg, format);
-    str = malloc(size);
-    if (str == NULL) {
-        va_end(arg);
-        /*
-         * Strictly speaking, GNU asprintf doesn't do this,
-         * but the caller isn't checking the return value.
-         */
-        message_OOM();
-        exit(1);
+    if (size < 0) {
+        return -1;
     }
-    rv = vsnprintf(str, size, format, arg);
+    str = malloc((size_t)size + 1);
+    if (str == NULL) {
+        return -1;
+    }
+    va_start(arg, format);
+    rv = vsnprintf(str, (size_t)size + 1, format, arg);
     va_end(arg);
 
     *string_ptr = str;
-    return (rv);
+    return rv;
 }
 
 #endif
@@ -315,7 +310,7 @@ static int resolveGid(const char *groupName, gid_t *pGid)
     return -1;
 }
 
-static int readModeUidGid(const char *configFile, int lineNum, char *key,
+static int readModeUidGid(const char *configFile, int lineNum, const char *key,
                           const char *directive, mode_t *mode, uid_t *pUid,
                           gid_t *pGid)
 {
@@ -815,21 +810,19 @@ int readAllConfigPaths(const char **paths)
 
 
     for (i = 0; i < defTabooCount; i++) {
-        int bytes;
         char *pattern = NULL;
 
         /* generate a pattern by concatenating star (wildcard) to the
          * suffix literal
          */
-        bytes = asprintf(&pattern, "*%s", defTabooExts[i]);
-        if (bytes != -1) {
-            tabooPatterns[i] = pattern;
-            tabooCount++;
-        } else {
+        if (asprintf(&pattern, "*%s", defTabooExts[i]) < 0) {
             free_2d_array(tabooPatterns, tabooCount);
             message_OOM();
             return 1;
         }
+
+        tabooPatterns[i] = pattern;
+        tabooCount++;
     }
 
     for (file = paths; *file; file++) {
@@ -943,12 +936,14 @@ static int globerr(const char *pathname, int theerr)
         newlog->what = NULL; \
     } while (0)
 #define RAISE_ERROR() \
-    if (newlog != defConfig) { \
-        state = STATE_ERROR; \
-        goto next_state; \
-    } else { \
-        goto error; \
-    }
+    do { \
+        if (newlog != defConfig) { \
+            state = STATE_ERROR; \
+            goto next_state; \
+        } else { \
+            goto error; \
+        } \
+    } while(0)
 #define MAX_NESTING 16U
 
 static int readConfigFile(const char *configFile, struct logInfo *defConfig)
@@ -969,6 +964,9 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
     static unsigned recursion_depth = 0U;
     char *globerr_msg = NULL;
     int in_config = 0;
+#ifdef HAVE_MADVISE
+    int r;
+#endif
     struct flock fd_lock = {
         .l_start = 0,
         .l_len = 0,
@@ -1011,7 +1009,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 
     if (getuid() == ROOT_UID) {
         if ((sb.st_mode & 07533) != 0400) {
-            message(MESS_DEBUG,
+            message(MESS_NORMAL,
                     "Potentially dangerous mode on %s: 0%o\n",
                     configFile, (unsigned) (sb.st_mode & 07777));
         }
@@ -1068,12 +1066,14 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
 
 #ifdef HAVE_MADVISE
 #ifdef MADV_DONTFORK
-    madvise(buf, length + 2,
-            MADV_SEQUENTIAL | MADV_WILLNEED | MADV_DONTFORK);
+    r = madvise(buf, length, MADV_SEQUENTIAL | MADV_WILLNEED | MADV_DONTFORK);
 #else /* MADV_DONTFORK */
-    madvise(buf, length + 2,
-            MADV_SEQUENTIAL | MADV_WILLNEED);
+    r = madvise(buf, length, MADV_SEQUENTIAL | MADV_WILLNEED);
 #endif /* MADV_DONTFORK */
+    if (r < 0) {
+        message(MESS_DEBUG, "Failed to advise use of memory: %s\n",
+                strerror(errno));
+    }
 #endif /* HAVE_MADVISE */
 
     message(MESS_DEBUG, "reading config file %s\n", configFile);
@@ -1092,8 +1092,16 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                 if (isalpha((unsigned char)*start)) {
                     free(key);
                     key = isolateWord(&start, &buf, length);
-                    if (key == NULL)
+                    if (key == NULL) {
+                        message(MESS_ERROR, "%s:%d failed to parse keyword\n",
+                                configFile, lineNum);
                         continue;
+                    }
+                    if (!isspace((unsigned char)*start)) {
+                        message(MESS_NORMAL, "%s:%d keyword '%s' not properly"
+                                " separated, found %#x\n",
+                                configFile, lineNum, key, *start);
+                    }
                     if (!strcmp(key, "compress")) {
                         newlog->flags |= LOG_FLAG_COMPRESS;
                     } else if (!strcmp(key, "nocompress")) {
@@ -1112,10 +1120,12 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         newlog->flags &= ~LOG_FLAG_SHAREDSCRIPTS;
                     } else if (!strcmp(key, "copytruncate")) {
                         newlog->flags |= LOG_FLAG_COPYTRUNCATE;
+                        newlog->flags &= ~LOG_FLAG_TMPFILENAME;
                     } else if (!strcmp(key, "nocopytruncate")) {
                         newlog->flags &= ~LOG_FLAG_COPYTRUNCATE;
                     } else if (!strcmp(key, "renamecopy")) {
                         newlog->flags |= LOG_FLAG_TMPFILENAME;
+                        newlog->flags &= ~LOG_FLAG_COPYTRUNCATE;
                     } else if (!strcmp(key, "norenamecopy")) {
                         newlog->flags &= ~LOG_FLAG_TMPFILENAME;
                     } else if (!strcmp(key, "copy")) {
@@ -1136,11 +1146,11 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         newlog->flags |= LOG_FLAG_DATEHOURAGO;
                     } else if (!strcmp(key, "dateformat")) {
                         freeLogItem(dateformat);
-                        newlog->dateformat = isolateLine(&start, &buf, length);
-                        if (newlog->dateformat == NULL)
-                            continue;
+                        newlog->dateformat = isolateValue(configFile, lineNum,
+                                                          key, &start, &buf,
+                                                          length);
                     } else if (!strcmp(key, "noolddir")) {
-                        newlog->oldDir = NULL;
+                        freeLogItem(oldDir);
                     } else if (!strcmp(key, "mailfirst")) {
                         newlog->flags |= LOG_FLAG_MAILFIRST;
                     } else if (!strcmp(key, "maillast")) {
@@ -1150,8 +1160,11 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         mode_t tmp_mode = NO_MODE;
                         free(key);
                         key = isolateLine(&start, &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            message(MESS_ERROR, "%s:%d failed to parse su option value\n",
+                                    configFile, lineNum);
+                            RAISE_ERROR();
+                        }
 
                         rv = readModeUidGid(configFile, lineNum, key, "su",
                                             &tmp_mode, &newlog->suUid,
@@ -1264,13 +1277,14 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         free(key);
                         key = isolateValue(configFile, lineNum, "shred cycles",
                                            &start, &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            RAISE_ERROR();
+                        }
                         newlog->shred_cycles = (int)strtoul(key, &chptr, 0);
                         if (*chptr || newlog->shred_cycles < 0) {
                             message(MESS_ERROR, "%s:%d bad shred cycles '%s'\n",
                                     configFile, lineNum, key);
-                            goto error;
+                            RAISE_ERROR();
                         }
                     } else if (!strcmp(key, "hourly")) {
                         set_criterium(&newlog->criterium, ROT_HOURLY, &criterium_set);
@@ -1305,8 +1319,9 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         free(key);
                         key = isolateValue(configFile, lineNum, "rotate count", &start,
                                            &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            RAISE_ERROR();
+                        }
                         newlog->rotateCount = (int)strtol(key, &chptr, 0);
                         if (*chptr || newlog->rotateCount < -1) {
                             message(MESS_ERROR,
@@ -1318,8 +1333,9 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         free(key);
                         key = isolateValue(configFile, lineNum, "start count", &start,
                                            &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            RAISE_ERROR();
+                        }
                         newlog->logStart = (int)strtoul(key, &chptr, 0);
                         if (*chptr || newlog->logStart < 0) {
                             message(MESS_ERROR, "%s:%d bad start count '%s'\n",
@@ -1330,8 +1346,9 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         free(key);
                         key = isolateValue(configFile, lineNum, "minage count", &start,
                                            &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            RAISE_ERROR();
+                        }
                         newlog->rotateMinAge = (int)strtoul(key, &chptr, 0);
                         if (*chptr || newlog->rotateMinAge < 0) {
                             message(MESS_ERROR, "%s:%d bad minimum age '%s'\n",
@@ -1342,8 +1359,9 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         free(key);
                         key = isolateValue(configFile, lineNum, "maxage count", &start,
                                            &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            RAISE_ERROR();
+                        }
                         newlog->rotateAge = (int)strtoul(key, &chptr, 0);
                         if (*chptr || newlog->rotateAge < 0) {
                             message(MESS_ERROR, "%s:%d bad maximum age '%s'\n",
@@ -1351,7 +1369,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                             RAISE_ERROR();
                         }
                     } else if (!strcmp(key, "errors")) {
-                        message(MESS_DEBUG,
+                        message(MESS_NORMAL,
                                 "%s: %d: the errors directive is deprecated and no longer used.\n",
                                 configFile, lineNum);
                     } else if (!strcmp(key, "mail")) {
@@ -1360,7 +1378,6 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                                         "mail", &start, &buf, length))) {
                             RAISE_ERROR();
                         }
-                        else continue;
                     } else if (!strcmp(key, "nomail")) {
                         freeLogItem(logAddress);
                     } else if (!strcmp(key, "missingok")) {
@@ -1421,7 +1438,6 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         }
 
                         while (*endtag) {
-                            int bytes;
                             char *pattern = NULL;
 
                             chptr = endtag;
@@ -1437,10 +1453,11 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                                     RAISE_ERROR();
                                 }
                                 tabooPatterns = tmp;
-                                bytes = asprintf(&pattern, "*%.*s", (int)(chptr - endtag), endtag);
+                                if (asprintf(&pattern, "*%.*s", (int)(chptr - endtag), endtag) < 0) {
+                                    message_OOM();
+                                    RAISE_ERROR();
+                                }
 
-                                /* should test for malloc() failure */
-                                assert(bytes != -1);
                                 tabooPatterns[tabooCount] = pattern;
                                 tabooCount++;
                             }
@@ -1481,7 +1498,6 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         }
 
                         while (*endtag) {
-                            int bytes;
                             char *pattern = NULL;
                             char **tmp;
 
@@ -1496,10 +1512,11 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                                 RAISE_ERROR();
                             }
                             tabooPatterns = tmp;
-                            bytes = asprintf(&pattern, "%.*s", (int)(chptr - endtag), endtag);
+                            if (asprintf(&pattern, "%.*s", (int)(chptr - endtag), endtag) < 0) {
+                                message_OOM();
+                                RAISE_ERROR();
+                            }
 
-                            /* should test for malloc() failure */
-                            assert(bytes != -1);
                             tabooPatterns[tabooCount] = pattern;
                             tabooCount++;
 
@@ -1515,8 +1532,9 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         free(key);
                         key = isolateValue(configFile, lineNum, "include", &start,
                                            &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            RAISE_ERROR();
+                        }
 
                         if (key[0] == '~' && key[1] == '/') {
                             /* replace '~' with content of $HOME cause low-level functions
@@ -1540,7 +1558,7 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                                 env_home = pwd->pw_dir;
                             }
 
-                            if (asprintf(&new_key, "%s/%s", env_home, key + 2) == -1) {
+                            if (asprintf(&new_key, "%s/%s", env_home, key + 2) < 0) {
                                 message_OOM();
                                 RAISE_ERROR();
                             }
@@ -1578,8 +1596,9 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         free(key);
                         key = isolateValue(configFile, lineNum, "extension name", &start,
                                            &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            RAISE_ERROR();
+                        }
                         freeLogItem (extension);
                         newlog->extension = key;
                         key = NULL;
@@ -1589,8 +1608,9 @@ static int readConfigFile(const char *configFile, struct logInfo *defConfig)
                         free(key);
                         key = isolateValue(configFile, lineNum, "addextension name", &start,
                                            &buf, length);
-                        if (key == NULL)
-                            continue;
+                        if (key == NULL) {
+                            RAISE_ERROR();
+                        }
                         freeLogItem (addextension);
                         newlog->addextension = key;
                         key = NULL;
@@ -1892,13 +1912,12 @@ duperror:
                                     continue;
                                 }
                             }
-                            ld = malloc(strlen(dirName) + strlen(newlog->oldDir) + 2);
-                            if (ld == NULL) {
+                            if (asprintf(&ld, "%s/%s", dirName, newlog->oldDir) < 0) {
                                 message_OOM();
                                 free(dirpath);
                                 goto error;
                             }
-                            sprintf(ld, "%s/%s", dirName, newlog->oldDir);
+
                             free(dirpath);
 
                             if (newlog->oldDir[0] != '/') {
@@ -2072,8 +2091,8 @@ duperror:
                 break;
             default:
                 message(MESS_FATAL,
-                        "%s: %d: readConfigFile() unknown state\n",
-                        configFile, lineNum);
+                        "%s: %d: readConfigFile() unknown state: %#x\n",
+                        configFile, lineNum, state);
         }
         if (*start == '\n') {
             lineNum++;
@@ -2093,12 +2112,14 @@ next_state: ;
 
     munmap(buf, length);
     close(fd);
+    free(globerr_msg);
     return logerror;
 error:
     /* free is a NULL-safe operation */
     free(key);
     munmap(buf, length);
     close(fd);
+    free(globerr_msg);
     return 1;
 }
 
