@@ -582,8 +582,12 @@ static int createOutputFile(const char *fileName, int flags, const struct stat *
         return -1;
     }
 
-    if ((sb_create.st_uid != sb->st_uid || sb_create.st_gid != sb->st_gid) &&
-            fchown(fd, sb->st_uid, sb->st_gid)) {
+    /* Only attempt to set user/group if running as root */
+    if (
+        ROOT_UID == geteuid() &&
+        (sb_create.st_uid != sb->st_uid || sb_create.st_gid != sb->st_gid) &&
+        fchown(fd, sb->st_uid, sb->st_gid)
+    ) {
         message(MESS_ERROR, "error setting owner of %s to uid %u and gid %u: %s\n",
                 fileName, (unsigned) sb->st_uid, (unsigned) sb->st_gid, strerror(errno));
         close(fd);
@@ -770,6 +774,8 @@ static int compressLogFile(const char *name, const struct logInfo *log, const st
     int error_printed = 0;
     char *prevCtx;
     pid_t pid;
+    int in_flags;
+    const char *in_how;
 
     message(MESS_DEBUG, "compressing log with: %s\n", log->compress_prog);
     if (debug)
@@ -785,8 +791,18 @@ static int compressLogFile(const char *name, const struct logInfo *log, const st
     compressedName = alloca(strlen(name) + strlen(log->compress_ext) + 2);
     sprintf(compressedName, "%s%s", name, log->compress_ext);
 
-    if ((inFile = open(name, O_RDWR | O_NOFOLLOW)) < 0) {
-        message(MESS_ERROR, "unable to open %s for compression: %s\n", name, strerror(errno));
+    in_flags = O_NOFOLLOW;
+    if (log->flags & LOG_FLAG_SHRED) {
+        /* need write access for shredding */
+        in_flags |= O_RDWR;
+        in_how = "read-write";
+    } else {
+        in_flags |= O_RDONLY;
+        in_how = "read-only";
+    }
+    if ((inFile = open(name, in_flags)) < 0) {
+        message(MESS_ERROR, "unable to open %s (%s) for compression: %s\n",
+            name, in_how, strerror(errno));
         return 1;
     }
 
@@ -1529,7 +1545,6 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
                               struct logState *state, struct logNames *rotNames)
 {
     struct tm now;
-    char *oldName = NULL;
     const char *compext = "";
     const char *fileext = "";
     int hasErrors = 0;
@@ -1567,7 +1582,7 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
     state->lastRotated = now;
 
     {
-        char *ld;
+        const char *ld;
         char *logpath = strdup(log->files[logNum]);
         if (logpath == NULL) {
             message_OOM();
@@ -1576,9 +1591,9 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
         ld = dirname(logpath);
         if (log->oldDir) {
             if (log->oldDir[0] != '/') {
-                rotNames->dirName =
-                    malloc(strlen(ld) + strlen(log->oldDir) + 2);
-                sprintf(rotNames->dirName, "%s/%s", ld, log->oldDir);
+                if (asprintf(&rotNames->dirName, "%s/%s", ld, log->oldDir) < 0) {
+                    rotNames->dirName = NULL;
+                }
             } else
                 rotNames->dirName = strdup(log->oldDir);
         } else
@@ -1770,11 +1785,8 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
                 sortGlobResult(&globResult, strlen(rotNames->dirName) + 1 + strlen(rotNames->baseName), dformat);
                 for (glob_count = 0; glob_count < globResult.gl_pathc && !hasErrors; glob_count++) {
                     struct stat sbprev;
+                    const char *oldName = globResult.gl_pathv[glob_count];
 
-                    if (asprintf(&oldName, "%s", (globResult.gl_pathv)[glob_count]) < 0) {
-                        message_OOM();
-                        return 1;
-                    }
                     if (stat(oldName, &sbprev)) {
                         if (errno == ENOENT)
                             message(MESS_DEBUG, "previous log %s does not exist\n", oldName);
@@ -1783,7 +1795,6 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
                     } else {
                         hasErrors = compressLogFile(oldName, log, &sbprev);
                     }
-                    free(oldName);
                 }
             } else {
                 message(MESS_DEBUG,
@@ -1793,6 +1804,7 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
             free(glob_pattern);
         } else {
             struct stat sbprev;
+            char *oldName;
             if (asprintf(&oldName, "%s/%s.%d%s", rotNames->dirName,
                          rotNames->baseName, logStart, fileext) < 0) {
                 message_OOM();
@@ -1810,15 +1822,6 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
         }
     }
 
-    /* adding 2 due to / and \0 being added by snprintf */
-    rotNames->firstRotated =
-        malloc(strlen(rotNames->dirName) + strlen(rotNames->baseName) +
-                strlen(fileext) + strlen(compext) + DATEEXT_LEN + 2 );
-    if (rotNames->firstRotated == NULL) {
-        message_OOM();
-        return 1;
-    }
-
     if (log->flags & LOG_FLAG_DATEEXT) {
         /* glob for compressed files with our pattern
          * and compress ext */
@@ -1862,16 +1865,14 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
             }
             if (mail_out != (size_t)-1) {
                 /* oldName is oldest Backup found (for unlink later) */
-                if (asprintf(&oldName, "%s", (globResult.gl_pathv)[mail_out]) < 0) {
-                    message_OOM();
-                    return 1;
-                }
+                const char *oldName = globResult.gl_pathv[mail_out];
                 rotNames->disposeName = strdup(oldName);
                 if (rotNames->disposeName == NULL) {
                     message_OOM();
+                    globfree(&globResult);
+                    free(glob_pattern);
                     return 1;
                 }
-                free(oldName);
             } else {
                 free(rotNames->disposeName);
                 rotNames->disposeName = NULL;
@@ -1882,14 +1883,21 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
             rotNames->disposeName = NULL;
         }
         /* firstRotated is most recently created/compressed rotated log */
-        sprintf(rotNames->firstRotated, "%s/%s%s%s%s",
+        if (asprintf(&rotNames->firstRotated, "%s/%s%s%s%s",
                 rotNames->dirName, rotNames->baseName, dext_str, fileext,
-                (log->flags & LOG_FLAG_DELAYCOMPRESS) ? "" : compext);
+                (log->flags & LOG_FLAG_DELAYCOMPRESS) ? "" : compext) < 0) {
+            message_OOM();
+            rotNames->firstRotated = NULL;
+            globfree(&globResult);
+            free(glob_pattern);
+            return 1;
+        }
         globfree(&globResult);
         free(glob_pattern);
     } else {
         int i;
         char *newName = NULL;
+        char *oldName;
 
         if (rotateCount == -1) {
             rotateCount = findLastRotated(rotNames, fileext, compext);
@@ -1911,13 +1919,19 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
             rotNames->disposeName = strdup(oldName);
             if (rotNames->disposeName == NULL) {
                 message_OOM();
+                free(oldName);
                 return 1;
             }
         }
 
-        sprintf(rotNames->firstRotated, "%s/%s.%d%s%s", rotNames->dirName,
+        if (asprintf(&rotNames->firstRotated, "%s/%s.%d%s%s", rotNames->dirName,
                 rotNames->baseName, logStart, fileext,
-                (log->flags & LOG_FLAG_DELAYCOMPRESS) ? "" : compext);
+                (log->flags & LOG_FLAG_DELAYCOMPRESS) ? "" : compext) < 0) {
+            message_OOM();
+            free(oldName);
+            rotNames->firstRotated = NULL;
+            return 1;
+        }
 
         for (i = rotateCount + logStart - 1; (i >= 0) && !hasErrors; i--) {
             free(newName);
@@ -1983,6 +1997,7 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
         if (asprintf(&(rotNames->finalName), "%s/%s%s%s", rotNames->dirName,
                      rotNames->baseName, dext_str, fileext) < 0) {
             message_OOM();
+            rotNames->finalName = NULL;
             return 1;
         }
         if (asprintf(&destFile, "%s%s", rotNames->finalName, compext) < 0) {
@@ -2001,6 +2016,7 @@ static int prerotateSingleLog(const struct logInfo *log, unsigned logNum,
         if (asprintf(&(rotNames->finalName), "%s/%s.%d%s", rotNames->dirName,
                      rotNames->baseName, logStart, fileext) < 0) {
             message_OOM();
+            rotNames->finalName = NULL;
         }
     }
 
@@ -2084,6 +2100,7 @@ static int rotateSingleLog(const struct logInfo *log, unsigned logNum,
                 free(rotNames->disposeName);
                 if (asprintf(&rotNames->disposeName, "%s%s", rotNames->finalName, ext) < 0) {
                     message_OOM();
+                    rotNames->disposeName = NULL;
                     return 1;
                 }
 
@@ -2212,11 +2229,6 @@ static int rotateLogSet(const struct logInfo *log, int force)
     struct logState **state;
     struct logNames **rotNames;
 
-    logHasErrors = calloc(log->numFiles, sizeof(int));
-    if (!logHasErrors) {
-        message_OOM();
-        return 1;
-    }
     message(MESS_DEBUG, "\nrotating pattern: %s ", log->pattern);
     if (force) {
         message(MESS_DEBUG, "forced from command line ");
@@ -2277,10 +2289,15 @@ static int rotateLogSet(const struct logInfo *log, int force)
 
     if (log->numFiles == 0) {
         message(MESS_DEBUG, "No logs found. Rotation not needed.\n");
-        free(logHasErrors);
         return 0;
     }
 
+    logHasErrors = calloc(log->numFiles, sizeof(int));
+    if (!logHasErrors) {
+        message_OOM();
+        return 1;
+    }
+
     if (log->flags & LOG_FLAG_SU) {
         if (switch_user(log->suUid, log->suGid) != 0) {
             free(logHasErrors);
