@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 . ./test-common.sh
 
@@ -10,9 +10,10 @@ preptest test.log 31 1 0
 
 $RLR test-config.31 --force
 
-stat -c %f test.log|grep 8180 >/dev/null
+$STAT_MODE_FORMAT test.log|grep 8180 >/dev/null
 if [ $? != 0 ]; then
-	echo "Bad mode of test.log, should be 0600"
+	echo "Bad mode of test.log, should be 0600, is:"
+	$STAT_MODE_FORMAT test.log
 	exit 3
 fi
 
