@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 . ./test-common.sh
 
@@ -12,7 +12,11 @@ preptest test.log 56 1 0
 DATE=""
 for i in $(seq 1 60)
 do
-    DATE=$(/bin/date "+%d-%m-%Y" --date "$i day ago" 2>/dev/null)
+    if date -v -1d > /dev/null 2>&1; then
+        DATE=$(date -v-${i}d "+%d-%m-%Y")
+    else
+        DATE=$(date "+%d-%m-%Y" --date "$i day ago")
+    fi
     echo "x" > test.log-$DATE
 done
 
