@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 . ./test-common.sh
 
@@ -6,7 +6,7 @@ cleanup 79
 
 # ------------------------------- Test 79 ------------------------------------
 preptest test.log 79 1
-$RLR test-config.79 -v --force >verbose.log
+$RLR test-config.79 --force >verbose.log
 if [ $? != 0 ]; then
 	echo "Logrotate exited with a non-zero exit code, but it should not have"
 	exit 3
