@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 . ./test-common.sh
 
@@ -8,7 +8,7 @@ cleanup 82
 preptest test.log 82 0
 
 for i in $(seq 32); do
-    $RLR test-config.82 -v --force
+    $RLR test-config.82 --force
 done
 
 test "$(ls test.log.* | wc -l)" -eq 32
