@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/bin/sh
 
 . ./test-common.sh
 
@@ -7,7 +7,7 @@ cleanup 83
 # ------------------------------- Test 83 ------------------------------------
 preptest test.log 83 1
 
-if $RLR test-config.83 -v --force; then
+if $RLR test-config.83 --force; then
     exit 1
 else
     exit 0
